#!/home/marin/anaconda3/bin/python
# -*- coding: utf-8 -*-

from utils import *

def figure_2C():
    # right Cauchy Green deformation tensor C = U^2, F=RU
    lam1,lam2,lam3 = sp.symbols("lambda_1 lamda_2 lambda_3")
    ce,k1,k2,m1,m2 = sp.symbols("c_e k_1 k_2 m_1 m_2")
    I1, I4, I6 = sp.symbols("I1 I4 I6")

    # strain energy function
    PSI = ce*(I1-3) + m1/(2*m2)*(sp.exp(m2*(I4-1)**2)-1) + k1/(2*k2)*(sp.exp(k2*(I6-1)**2)-1)

    # compute stresses
    dpsidI1 = PSI.diff(I1)
    # print(dpsidI1.simplify())

    dpsidI4 = PSI.diff(I4)
    # print(dpsidI4.simplify())

    dpsidI6 = PSI.diff(I6)
    # print(dpsidI6.simplify())

    # compute the final stress functions
    p_h = 2*dpsidI1*lam3**2
    sigma_1_h = -p_h + 2*dpsidI1*lam1**2 + I4*dpsidI4*1 + I6*dpsidI6*0
    sigma_2_h = -p_h + 2*dpsidI1*lam2**2 + I4*dpsidI4*0 + I6*dpsidI6*1
    # print(p_h.simplify())
    # print(sigma_1_h.simplify())
    # print(sigma_2_h.simplify())

    # make callable functions
    holzapfel_s1 = sp.lambdify([lam1,lam2,lam3,I1,I4,I6,ce,m1,m2,k1,k2], sigma_1_h)
    holzapfel_s2 = sp.lambdify([lam1,lam2,lam3,I1,I4,I6,ce,m1,m2,k1,k2], sigma_2_h)


    # plot the fitted stresses
    N = 256
    lam = np.array([np.linspace(1.0,1.18,N),
                    np.linspace(1.0,1.18,N),
                    np.zeros(N)])
    lam[2] = 1./np.product(lam[:2],axis=0)

    I1 = np.sum(lam**2, axis=0)
    I4 = lam[0]**2
    I6 = lam[1]**2

    # parameters from Jupyter notebook
    ce = 2.0
    m1 = 33.51315387; m2 = 13.47707236
    k1 = 102.73683109; k2 = 12.3041447
    
    # generate the data
    s1 = holzapfel_s1(lam[0],lam[1],lam[2],I1,I4,I6,ce,m1,m2,k1,k2)
    s2 = holzapfel_s2(lam[0],lam[1],lam[2],I1,I4,I6,ce,m1,m2,k1,k2)

    #figure
    fig, ax = plt.subplots(1, 2, figsize=cm(18,7), gridspec_kw={"width_ratios": [1,0.5]})

    # plot model data
    p=ax[0].plot((lam[0]-1),s1,label="Spanwise Fiber")
    ax[0].plot((lam[0]-1),s2,label="Chordwise Fiber")

    # gradient at origin
    grad = (s1[1]-s1[0]) / (lam[0,1]-lam[0,0])
    print(grad)

    # plot linear fit at the origin
    ax[0].plot(np.linspace(0,0.16), grad*np.linspace(0,0.16), "-.C0", lw=0.75, alpha=0.5, label="Tangent Stiffness")


    # plot experimental data
    data = np.loadtxt("data/experimental_constitutive_curves_bat_PNAS.txt")
    e1,stress1,e2,stress2 = data
    
    ax[0].plot(e1, stress1, ".C0", alpha=0.25, label="Experimental Spanwise")
    ax[0].plot(e2, stress2, ".C1", alpha=0.25, label="Experimental Chordwise")
    ax[0].set_xticks(np.linspace(0,0.2,5))
    ax[0].set_xlim(0,0.2)
    ax[0].set_ylim(0,100)
    ax[0].set_xlabel(r"$e_{11}$, $e_{22}$")
    ax[0].set_ylabel(r"$\sigma_{11}$, $\sigma_{22}$ (kPa)")
    ax[0].legend()


    # get element names
    f = open("data/Bat_elements.nam")
    elements=[]; node=[]
    for line in f:
        if line[7:13]=="ELSET=":
            el = line[13:].strip()
            continue
        for item in line.strip().split():
            elements.append(el)
            node.append(int(item.replace(",","")))
    f.close()
    elements = np.array(elements)
    node = np.array(node)
    idx = np.argwhere(elements!='BONES')

    # get elements
    f = open("data/Bat_geom.inp")
    lines = f.readlines()[1:]
    ids=[]; x=[]; y=[]; z=[]
    for k in range(len(lines)):
        if lines[k].strip()[0:3]=="*EL":
            finish = k; break
    f.close()
    conectiv = []
    for k in range(0,finish):
        ids_,x_,y_,z_ = to_float(lines[k].strip().replace(",","").split())
        ids.append(ids_)
        x.append(x_)
        y.append(y_)
        z.append(z_)
    for k in range(finish+1,len(lines)):
        n,a,b,c = to_int(lines[k].strip().replace(",","").split())
        conectiv.append([a-2,b-2,c-2])
    conectiv = np.array(conectiv)
    x = np.array(x)
    y = np.array(y)
    z = np.array(z)

    # scale fo the arrows
    scale = 30
    L = 27.5*2

    mask = elements!="BONES"
    trian = tri.Triangulation(x/L, y/L, triangles=conectiv, mask=mask)
    # ax[1].triplot(trian, color="#7A695A", lw=0.5, alpha=1)
    ax[1].tricontourf(trian, np.ones_like(trian.x), colors="#7A695A")

    for el,col in zip(["MEMBRANE1","MEMBRANE2"],
                      ["#454444","#666666"]):
        mask = elements!=el
        trian = tri.Triangulation(x/L, y/L, triangles=conectiv, mask=mask)
        ax[1].triplot(trian, color=col, lw=0.3, alpha=0.5)
        
        # inner membrane
        if el == "MEMBRANE1":
            print("Membrane 1:")
            alpha = 22.5
        else: 
            print("Membrane 2:")
            alpha = 45.0

        for a1,a2,col in zip([np.cos(np.radians(alpha)),np.cos(np.radians(alpha+90))],
                                [np.sin(np.radians(alpha)),np.sin(np.radians(alpha+90))],
                                ["C1","C0"]):
            print("Direction cosine fibre 1: %.6f, %.6f (%.6f)" % (a1,a2,1-a1**2-a2**2))
            
            # make grid
            x_min, x_max = min(trian.x),max(trian.x)
            y_min, y_max = min(trian.y),max(trian.y)
            grid_x, grid_y = np.mgrid[x_min:x_max:128j, y_min:y_max:128j]

            # make interpolation function
            interpolator1 = tri.CubicTriInterpolator(trian, a1*np.ones_like(x), kind='geom')
            interpolator2 = tri.CubicTriInterpolator(trian, a2*np.ones_like(x), kind='geom')
            
            # interpolate
            e1 = interpolator1(grid_x, grid_y)
            e2 = interpolator2(grid_x, grid_y)

            # stremalines
            # q1 = ax[1].quiver(grid_x, grid_y, e1, e2,scale=scale,alpha=0.9,color=col)
            ax[1].streamplot(grid_x.T, grid_y.T, e1.T, e2.T, color=col, linewidth=0.7)

    # remove arrows
        for art in ax[1].get_children():
            if not isinstance(art, matplotlib.patches.FancyArrowPatch):
                continue
            art.remove()

    ax[0].spines[['right', 'top']].set_visible(False)
    ax[0].get_xaxis().tick_bottom()
    ax[0].get_yaxis().tick_left()
    ax[0].minorticks_off()

    ax[1].axis("off")
    ax[1].set_xlim(min(x/L),max(x/L))
    ax[1].set_ylim(min(y/L),max(y/L))
    ax[1].set_aspect("equal","box")

    for fmt in ["png","svg"]:
        plt.savefig("figures/PNAS_figure_2C."+fmt, dpi=600 ,bbox_inches="tight", transparent=False)
    plt.show()


def figure_4E():
    
    Cauchy = [0.05,0.5,0.05,0.5]
    frequency = 0.5

    # figure
    fig = plt.figure(1, figsize=cm(19.6,14.6))
    ax = fig.subplots(2,2)
    ax = ax.ravel()

    coaxiality_store = []

    for ix,Ca in enumerate(Cauchy):
        fname  = "data/Bat_fr%.4f_Ca%.4f" % (frequency, Ca)
        if ix in [2,3]:
            fname += "_fibre"
            
        print(fname)
        
        # load file and reshape
        data = np.loadtxt(fname+"_solid.txt")
        x = data[0]; y=data[1]; z=data[2]
        triangles = data[3].reshape((3143, 3)).astype(int)
        s_all = np.reshape(data[4:10],(6, 9429))
        e_all = np.reshape(data[10:],(6, 9429))

        # make triangles
        trian  = tri.Triangulation(x, y ,triangles=triangles)

        # find unique triangle and nodes
        coords = np.array([x,y,z])
        points_new,unique = np.unique(coords, axis=1, return_index=True)
        triangles_new = np.zeros_like(triangles)
        for i in range(triangles.shape[0]):
            ids = triangles[i,:]
            ids_new = np.zeros(3)
            for j in range(3):
                r = sum(coords[:,ids[j]]**2)
                idx = np.where((r-np.sum(points_new**2,axis=0))==0.0)[0]
                ids_new[j] = min(idx)
            triangles_new[i,:] = ids_new
        
        # make triangles
        trian_new  = tri.Triangulation(x[unique], y[unique] ,triangles=triangles_new)

        #storage for strain
        e_avrg = e_all[:,unique]
        e_principal = np.zeros((3,e_avrg.shape[1]))
        e_basis = np.zeros((3,3,e_avrg.shape[1]))

        # storage for stress
        s_avrg = s_all[:,unique]
        s_principal = np.zeros((3,s_avrg.shape[1]))
        s_basis = np.zeros((3,3,s_avrg.shape[1]))

        for i in range(e_principal.shape[1]):
            
            # compute principal strain
            lam, phi = compute_principal(e_avrg[:,i])
            e_principal[:,i] = lam
            e_basis[:,:,i] = phi

            # principal stress
            lam, phi = compute_principal(s_avrg[:,i])
            s_principal[:,i] = lam
            s_basis[:,:,i] = phi

        # use only unique points
        trian = trian_new
        x_min, x_max = min(trian.x),max(trian.x)
        y_min, y_max = min(trian.y),max(trian.y)

        # make sure the vector field point globaly in the same direction, issue due to the
        e_mean  = np.mean(e_basis[:2,0,:], axis=-1)
        s_mean  = np.mean(s_basis[:2,0,:], axis=-1)
        # eigenvalue unique up to a -1 factor
        e_basis = np.where(np.einsum('ij,i->j', e_basis[:2,0,:], e_mean)<0, -e_basis, e_basis)
        s_basis = np.where(np.einsum('ij,i->j', s_basis[:2,0,:], s_mean)<0, -s_basis, s_basis)

        
        # prepare interpolation
        interpolator1 = tri.CubicTriInterpolator(trian, e_basis[0,0,:], kind='min_E')
        interpolator2 = tri.CubicTriInterpolator(trian, e_basis[1,0,:], kind='min_E')
        interpolator3 = tri.CubicTriInterpolator(trian, e_principal[0,:], kind='min_E')

        # make grid
        grid_x, grid_y = np.mgrid[x_min:x_max:32j, y_min:1.1*y_max:32j]

        # interpolate
        grid_e1 = interpolator1(grid_x, grid_y)
        grid_e2 = interpolator2(grid_x, grid_y)
       
        # plot fibre direction
        ax[ix], elements = get_fibre_direction(ax[ix])

        # scale line width and color with strain
        e_mag = interpolator3(grid_x, grid_y)
        e_max = 0.1
        e_width = (e_mag-np.min(e_mag))/(e_max-np.min(e_mag))+0.6

        # coaxiality of principal strain/stress
        dot = abs(np.einsum('ij,ij->j', e_basis[:,0,:], s_basis[:,0,:]))
        coaxiality_store.append(abs(dot))

        # plot triangles
        trian_new = tri.Triangulation(y[unique], x[unique] ,triangles=triangles_new) #, mask=(elements=="BONES"))
        ax[ix].triplot(trian_new, color="gray", linewidth=0.4)

        # contour of coaxility
        contour = ax[ix].tricontourf(trian_new, abs(dot), cmap="viridis", levels=np.linspace(0.,1.,31))

        # back dashed contour at C=0.75
        ax[ix].tricontour(trian_new, abs(dot), colors="k", levels=[0.75], linewidths=1.0,
                        linestyles="dashdot")
        
        # custom strain color map so that we dont get white at all
        orig_cmap = plt.cm.Reds
        colrs = orig_cmap(np.linspace(0.3, 1.0, 10))
        cmap = matplotlib.colors.LinearSegmentedColormap.from_list("mycmap", colrs)

        #  plot principal strain streamlines
        stream1 = ax[ix].streamplot(grid_y[0,:], grid_x[:,0], grid_e2, grid_e1, color=e_mag, cmap=cmap, norm=colors.Normalize(0,e_max,True), #="r",
                                    linewidth=e_width, density=0.9, arrowsize=0.1, broken_streamlines=False)
        stream1.lines.set_alpha(1.0)

        # add the colorbar 
        pos_cax2 = [0.05, -0.03, 0.5, 0.03]
        if ix==2:
            cax2 = ax[ix].inset_axes(pos_cax2)
            cbar = plt.colorbar(contour, ax=ax[ix], cax=cax2, ticks=np.linspace(0,1,6), label="Coaxiality", orientation='horizontal')
            cbar.set_ticks([0.,0.5,1.0])
            cax2 = ax[ix].inset_axes([0.6, -0.03, 0.5, 0.03])
            cbar = plt.colorbar(stream1.lines, ax=ax[ix], cax=cax2, label="Max. Principal Strain", orientation='horizontal')
            cbar.set_ticks([0.,0.05, 0.10])
       
        # remove arrows from the streamlines
        for art in ax[ix].get_children():
            if not isinstance(art, matplotlib.patches.FancyArrowPatch):
                continue
            art.remove()

        # save only a portion of that figure for later
        if ix!=4:
            ax[ix].axis("off")
            ax[ix].invert_yaxis()
            ax[ix].set_aspect("equal","box")
            ax[ix].set_xlim(0.9*y_min,1.1*y_max)
            ax[ix].set_ylim(1.1*x_max,1.1*x_min)
        extent = ax[ix].get_window_extent().transformed(fig.dpi_scale_trans.inverted())
        fig.savefig('figures/pxp_%d.png' % ix, bbox_inches=extent.expanded(0.95, 0.95))
    
    labels = [r"$Ca=0.05$",
              r"$Ca=0.5$",
              r"Fiber ($Ca=0.05$)",
              r"Fiber ($Ca=0.5$)"]

    for i,a in enumerate(ax):
        a.text(0.2, 0.1, labels[i], transform=a.transAxes,
               va='center', ha='left', fontsize=10) 
    plt.tight_layout()
    for fmt in ["png","svg"]:
        fig.savefig("figures/PNAS_figure_4E."+fmt, dpi=600, bbox_inches="tight")
    plt.show()



def figure_4():

    # frequency
    frequency = 0.5

    # make figure
    fig1 = plt.figure(1, figsize=cm(19.8,10))
    gs = fig1.add_gridspec(nrows=4,ncols=3,width_ratios=[1.5,1,1.5])
    ax_1 = fig1.add_subplot(gs[:2,0])
    ax_2 = fig1.add_subplot(gs[0,1])
    ax_3 = fig1.add_subplot(gs[1,1],sharex=ax_2)
    ax_6 = fig1.add_subplot(gs[2,1],sharex=ax_2)
    ax_7 = fig1.add_subplot(gs[3,1],sharex=ax_2)
    ax_4 = fig1.add_subplot(gs[0,2])
    ax_5 = fig1.add_subplot(gs[2:,0])
    ax_8 = fig1.add_subplot(gs[2:,2])
    ax = np.array([ax_2,ax_3,ax_6,ax_7,ax_5,ax_1,ax_4,ax_8])

    # colors and line style
    color=["C1","C0","k","C2"]
    ls = ["-",(0,(5,1)),(0,(3,1,1,1)),(0,(5,1,1,1,1,1))]

    # loop over some Cauchy
    for ij,Ca in enumerate([5.0,0.5,0.05,0.05]):
        
        fname = "data/Bat_fr%.4f_Ca%.4f" % (frequency, Ca)
        print("Working in:")
        if ij==3:
            fname = fname+"_fibre"
        print(" "+fname)
        y_t = []
        
        fname = "data/trailing_edge_position_fr%.4f_Ca%.4f" % (frequency, Ca)
        data = np.loadtxt(fname+".txt")
        xs = data[0].reshape((200,256))
        ys = data[1].reshape((200,256))

        for k in range(0,ys.shape[0],1):

            x_ = xs[k,:]
            y_ = ys[k,:]
            y_t.append(y_[len(y_)//2])

            # plot membrane deflection as line
            if k%2==0:
                ax[ij].plot(x_, y_ , color[ij], lw=0.5, alpha=0.5)

        # # select point in middle
        T = 1./frequency/len(y_t)
        y_t_dot = np.gradient(y_t, edge_order=2)/T
        f, y_tilde = signal.periodogram(y_t_dot, 1./T, 'hamming')
    
        # plot spectrum on same axis
        label = r"$Ca: %.2f$" % Ca
        if ij==3:
            label = "Fiber\n"+"("+label+")"
        ax[6].loglog(f, abs(y_tilde)/len(y_tilde), color[ij], lw=0.5+0.25*ij, label=label)

    pparam = dict(xlim=(0,1), ylim=(-0.15,0.25), ylabel="$h/L$")
    for ai in ax[:4]:
        ai.set(**pparam)
        ai.set_aspect("equal", "box")
    ax[3].set_xlabel("$x/L$")

    for ai in ax[:]:  
        ai.spines[['right', 'top']].set_visible(False)
        ai.get_xaxis().tick_bottom()
        ai.get_yaxis().tick_left()
        ai.minorticks_off()

    #  load efficiencies
    data = np.loadtxt("data/efficiencies_various_cauchy.txt")
    Cauchy, thrust, lift, power, efficiencies, efficiencies_lift =  data
    
    l1,=ax[4].semilogx(Cauchy, efficiencies, '-sk',lw=1,alpha=1, ms=4.,markeredgewidth=0.75,mfc="None")
    l2,=ax[4].semilogx(Cauchy, efficiencies_lift, "--xk", lw=1,alpha=0.75,ms=4.,markeredgewidth=0.75,mfc="None")

    #updated 11/04/23
    data = np.loadtxt("data/efficiencies_various_strouhal_fibres.txt")
    Ca, efficiency, rankine = data

    Ca = np.flipud(Cauchy)
    # print(Ca, efficiency, rankine)
    l3,=ax[4].plot(Ca, efficiency, '-sC2', lw=1.2,alpha=1,ms=4.,markeredgewidth=0.75, mfc="None")
    ax[4].plot(Ca, rankine, '--xC2', lw=1.2,alpha=0.75,ms=4.,markeredgewidth=0.75, mfc="None")

    l1,=ax[4].plot([100,101], [0.,0.], '-k',lw=1,alpha=1, ms=4.,markeredgewidth=0.75,mfc="None")
    l3,=ax[4].plot([100,101], [0.,0.], '-C2', lw=1.2,alpha=1,ms=4.,markeredgewidth=0.75, mfc="None")

    # make nice
    pparam = dict(xlabel="Cauchy Number", xlim=(0.02,6), ylim=(0.2,0.5))
    lab ="$\eta$"
    ax[4].set(**pparam)
    ax[4].set_ylabel(lab)
    ax[4].set_yticks(np.linspace(0.2,0.5,4))
    ax[4].legend([l1,l3], ["Linear isotropic", "Fiber-reinforced"], loc=3)

    ax1 = ax[4].twinx()
    l1,=ax1.plot([10,11],[1,1],'-ok', lw=1.2,alpha=1,ms=4.,markeredgewidth=0.75, mfc="None")
    l2,=ax1.plot([10,11],[1,1],'--xk', lw=1.2,alpha=0.75,ms=4.,markeredgewidth=0.75, mfc="None")
    ax1.axis("off")
    ax1.legend([l1,l2], [r"$\eta_{{P}}$",r"$\eta_{{RF}}$"], loc=2)

    #  load efficiencies
    ls = ["C0","C1"]; Cas = [0.5,5.0]; leg=[]
    for i in range(2):
        data = np.loadtxt("data/efficiencies_various_strouhal_Ca%.4f.txt" % Cas[i])
        Strouhal, efficiencies, efficiencies_lift = data
        # plotfor a Cauchy number
        l1,=ax[5].plot(Strouhal, efficiencies, "-s"+ls[i],lw=1,alpha=1,ms=4.,markeredgewidth=0.75,mfc="None")
        l2,=ax[5].plot(Strouhal, efficiencies_lift, "--x"+ls[i],lw=1,alpha=0.5,ms=4.,markeredgewidth=0.75,mfc="None")
        leg.append([l1,l2])

    # make nice efficiency plot (strouhal)
    pparam = dict(xlabel=r"Strouhal Number", ylabel=r"$\eta$", xlim=(0.1,0.8), ylim=(0.2,0.5))
    ax[5].set(**pparam)
    ax[5].set_yticks(np.linspace(0.2,0.5,4))
    ax[5].legend(leg[0], [r"$\eta_{{P}}$",r"$\eta_{{RF}}$"], loc=2)

    # dummy line for proper legend
    ax2 = ax[5].twinx()
    l1,=ax2.plot([10,11],[1,1],ls[0],lw=1)
    l2,=ax2.plot([10,11],[1,1],ls[1],lw=1)
    ax2.axis("off")
    ax2.legend([l1,l2], [r"$Ca=0.5$", r"$Ca=5.0$"], loc=3)

    # spectrum plot
    pparam = dict(xlim=(0.5e0,1e3), ylim=(1e-6,1e0), 
                    xlabel=r"$f/f_0$", ylabel=r"$\mathcal{PSD}(\dot{h}/L)$")
    ax[-2].set(**pparam)
    ax[-2].legend()

    # no need of last axis
    ax[-1].remove()

    plt.tight_layout()
    for fmt in ["png","svg"]:
        fig1.savefig("figures/PNAS_Figure_4."+fmt, bbox_inches="tight", dpi=600)
    plt.show()



def figure_1B():

    # load data 
    df = pd.read_csv(r"data/bat_wing_strouhal.csv")

    # make latex table
    make_table(df)
    # get data of interest
    species = np.flipud(np.array([r"\emph{"+df.iloc[i]["Unnamed: 2"]+"}" for i in range(1,24)]))
    strouhal_h = np.flipud(np.array([float(df.iloc[i]["Strouhal number"]) for i in range(1,24)]))
    strouhal_l = np.flipud(np.array([float(df.iloc[i]["Unnamed: 17"]) for i in range(1,24)]))
    vel_l = np.flipud(np.array([float(df.iloc[i]["Speed range"]) for i in range(1,24)]))
    vel_h = np.flipud(np.array([float(df.iloc[i]["Unnamed: 11"]) for i in range(1,24)]))
    L_mean = np.flipud(np.array([float(df.iloc[i]["Wing span"]) for i in range(1,24)]))
    f_mean = np.flipud(np.array([float(df.iloc[i]["Wingbeat frequency"]) for i in range(1,24)]))
    f_variance = np.flipud(np.array([float(df.iloc[i]["Unnamed: 13"]) for i in range(1,24)]))
    theta_mean = np.flipud(np.array([float(df.iloc[i]["Wingbeat amplitude"]) for i in range(1,24)]))
    theta_variance = np.flipud(np.array([float(df.iloc[i]["Unnamed: 15"]) for i in range(1,24)]))
    
    # generate mean strouhal number
    mean = np.mean(np.vstack((strouhal_l,strouhal_h)), axis=0)
    variance = strouhal_h - mean

    #uncertainty in amplitude absolute and relative
    U_theta = np.radians(theta_mean) - np.radians(theta_mean+theta_variance)
    UA_theta = U_theta/theta_mean

    # uncertainty in frequency
    UA_f = f_variance / f_mean
    # uncertainty in Strouhal
    UA_st = UA_theta + UA_f

    # half the span
    mean_st = f_mean*0.5*L_mean*np.radians(theta_mean)/np.mean(np.vstack((vel_l,vel_h)), axis=0)
    
    # plot stuff
    x_positions = np.arange(len(species))
    fig, ax = plt.subplots(1,1,figsize=cm(14,18), dpi=128)
    
    for i in range(len(x_positions)):
        mu = mean[i]
        sigma = np.sqrt(variance[i])/3
        x = np.linspace(mu - 3*sigma, mu + 3*sigma, 100)
        dist = stats.norm.pdf(x, mu, sigma)
        dist /= max(abs(dist))/1.5 # normalize
        ax.plot(x, x_positions[i]+dist, "C0", lw=1.5, alpha=0.4)
        ax.fill_between(x, x_positions[i],x_positions[i]+dist, edgecolor="None", color="C0", alpha=0.1)
    
    # plot common range
    ax.fill_betweenx(x_positions, 0.25*np.ones_like(x_positions), 0.35*np.ones_like(x_positions), edgecolor="None", color="k", alpha=0.1, zorder=-100)
    
    # tidy
    ax = make_axis_nice(ax)
    ax.set_yticks(x_positions)
    ax.set_yticklabels(species, fontsize=11)
    ax.set_xlim(0.,1.5)
    plt.minorticks_off()
    plt.tight_layout()
    for fmt in ["png","svg"]:
        plt.savefig("figures/PNAS_figure_1B."+fmt, format=fmt, bbox_inches="tight", dpi=600)
    plt.show()
    return None


def figure_2D():

    # load data from experiments and make it periodic
    YZ = np.genfromtxt("data/YZ.csv", delimiter=","); YZ = np.flipud(np.vstack((YZ,YZ[0,:])))*10
    XZ = np.genfromtxt("data/XZ.csv", delimiter=","); XZ = np.vstack((XZ,XZ[0,:]))*10
    XY = np.genfromtxt("data/YX.csv", delimiter=","); XY = np.flipud(np.vstack((XY,XY[0,:])))*10
    YZ_t = np.genfromtxt("data/YZ_t.csv", delimiter=","); YZ_t = np.flipud(np.vstack((YZ_t,YZ_t[0,:])))*10
    XZ_t = np.genfromtxt("data/XZ_t.csv", delimiter=","); XZ_t = np.vstack((XZ_t,XZ_t[0,:]))*10
    YX_t = np.genfromtxt("data/YX_t.csv", delimiter=","); YX_t = np.flipud(np.vstack((YX_t,YX_t[0,:])))*10

    # load data
    data = np.loadtxt("data/wing_motion_simulation.txt")
    x = data[[0,3]]
    y = data[[1,4]]
    z = data[[2,5]]

    ax = plt.figure(figsize=cm(18,12)).add_subplot(projection='3d')
    for id,lab,col in zip([0,1],["carpus","wing tip"],["C3","C2"]):
        ax.plot(x[id,:]-0.15,y[id,:],z[id,:],col,lw=2)
        ax.plot(y[id,:],z[id,:],zs=-0.5,zdir="x",color=col,ls="--")
        ax.plot(x[id,:]-0.15,z[id,:],zs=-0.1,zdir="y",color=col,ls="-.")
        ax.plot(x[id,:]-0.15,y[id,:],zs=-0.5, zdir='z',color=col,ls=(0, (1, 1)))

    ax.plot(YZ[:,0],YZ[:,1],'sC2',zs=-0.5,zdir="x",mfc="None",ms=3,markeredgewidth=0.5)
    ax.plot(XZ[:,0],XZ[:,1],'sC2',zs=-0.1,zdir="y",mfc="None",ms=3,markeredgewidth=0.5)
    l1,=ax.plot(XY[:,1],XY[:,0],'sC2',zs=-0.5,zdir="z",mfc="None",ms=3,markeredgewidth=0.5)

    ax.plot(YZ_t[:17,0],YZ_t[:17,1],"oC3",zs=-0.5,zdir="x",mfc="None", ms=3, markeredgewidth=0.5)
    ax.plot(XZ_t[:17,0],XZ_t[:17,1],"oC3",zs=0.0,zdir="y",mfc="None", ms=3, markeredgewidth=0.5)
    l2,=ax.plot(YX_t[1:14,1],YX_t[1:14,0],"oC3",zs=-0.5,zdir="z",mfc="None", ms=3, markeredgewidth=0.5)

    ax.legend(handles=[l1,l2], labels=["wingtip","carpus"], fontsize=10)

    ax.set_xlabel(r"$X/L$", fontsize=12)
    ax.set_ylabel(r"$Y/L$", fontsize=12)
    ax.set_zlabel(r"$Z/L$", fontsize=12)
    ax.set_xlim(-0.5,0.5)
    ax.set_ylim(-1.2,0.0)
    ax.set_zlim(-0.5,0.5)
    ax.set_yticks([]); ax.set_xticks([])
    ax.set_yticks([0.,-0.6,-1.2])
    ax.set_xticks([-0.5,0.0,0.5])
    ax.set_zticks(np.linspace(-0.5,0.5,3))
    # ax.view_init(elev=31, azim=-28)
    ax.view_init(elev=21, azim=-60)

    # Remove grid lines
    ax.xaxis._axinfo["grid"].update({"linewidth":0.1, "color":"#bfbfbf"})
    ax.yaxis._axinfo["grid"].update({"linewidth":0.1, "color":"#bfbfbf"})
    ax.zaxis._axinfo["grid"].update({"linewidth":0.1, "color":"#bfbfbf"})

    # custom color
    ax.w_xaxis.set_pane_color((1.0, 1.0, 1.0, 0.0))
    ax.w_yaxis.set_pane_color((1.0, 1.0, 1.0, 0.0))
    ax.w_zaxis.set_pane_color((1.0, 1.0, 1.0, 0.0))

    ax.set_box_aspect((1,1.2,1))
    for fmt in ["png","svg"]:
        plt.savefig("figures/PNAS_figure_2D."+fmt, format=fmt, dpi=600, bbox_inches="tight")
    plt.show()


if __name__=="__main__":
    # plot
    figure_1B()
    figure_2C()
    figure_2D()
    figure_4()
    figure_4E()
